@@ -2,19 +2,20 @@
 
 import os
 import random
-from typing import Dict, Any
+from typing import Dict, Any, List
 
 from django.conf import settings
 from django.core.handlers.wsgi import WSGIRequest
 from django.db import transaction
 from django.http import HttpResponseBadRequest
 from django.http import HttpResponseRedirect
-from django.http.response import HttpResponse
+from django.http.response import HttpResponse, JsonResponse
 from django.shortcuts import render
-from django.urls import reverse
+from django.urls import reverse, URLPattern
 
 import core.models as models
 from core.lights.lights import Lights
+from core.api import Api
 from core.musiq.musiq import Musiq
 from core.network_info import NetworkInfo
 from core.settings.settings import Settings
@@ -26,11 +27,13 @@ class Base(Stateful):
     """This class contains methods that are needed by all pages."""
 
     def __init__(self) -> None:
+        self.urlpatterns: List[URLPattern] = []
         self.settings = Settings(self)
         self.user_manager = UserManager(self)
         self.lights = Lights(self)
         self.musiq = Musiq(self)
         self.network_info = NetworkInfo(self)
+        self.api = Api(self)
 
     def start(self) -> None:
         self.lights.start()
@@ -66,6 +69,7 @@ class Base(Stateful):
         Increments the visitors counter."""
         self._increment_counter()
         return {
+            "base_urls": self.urlpatterns,
             "voting_system": self.settings.basic.voting_system,
             "hashtag": self._get_random_hashtag(),
             "controls_enabled": self.user_manager.has_controls(request.user),
@@ -117,3 +121,15 @@ class Base(Stateful):
         if self.user_manager.is_admin(request.user):
             return HttpResponseRedirect(reverse("settings"))
         return HttpResponseRedirect(reverse("musiq"))
+
+    @Lights.option
+    def set_lights_shortcut(self, request: WSGIRequest) -> None:
+        return self.lights.controller._set_lights_shortcut(request)
+
+    @Settings.option
+    def upgrade_available(self, _request: WSGIRequest) -> HttpResponse:
+        latest_version = self.settings.system._fetch_latest_version()
+        current_version = settings.VERSION
+        if latest_version and latest_version != current_version:
+            return JsonResponse(True, safe=False)
+        return JsonResponse(False, safe=False)
