@@ -52,7 +52,6 @@ def disabled_when_voting(func: Callable) -> Callable:
         ):
             return HttpResponseForbidden()
         func(self, request, *args, **kwargs)
-        self.musiq.update_state()
         return HttpResponse()
 
     return wraps(func)(_decorator)
@@ -211,6 +210,15 @@ class Controller:
                 if allowed:
                     self.playback.player.mixer.set_volume(round(self.volume * 100))
 
+    @disabled_when_voting
+    @control
+    def shuffle_all(self, request: WSGIRequest) -> HttpResponse:
+        """Shuffles the queue. Only admin is permitted to do this."""
+        if not self.musiq.base.user_manager.is_admin(request.user):
+            return HttpResponseForbidden()
+        self.playback.queue.shuffle()
+        return HttpResponse()
+
     @disabled_when_voting
     @control
     def remove_all(self, request: WSGIRequest) -> HttpResponse:
