@@ -11,7 +11,7 @@ from django.forms.models import model_to_dict
 from django.http import HttpResponseBadRequest
 from django.http.response import HttpResponse, JsonResponse
 from django.shortcuts import render
-from django.views.decorators.csrf import csrf_exempt
+from django.urls import URLPattern
 
 import core.musiq.song_utils as song_utils
 from core.models import CurrentSong
@@ -22,10 +22,7 @@ from core.musiq.music_provider import MusicProvider, WrongUrlError, ProviderErro
 from core.musiq.song_provider import SongProvider
 from core.musiq.playlist_provider import PlaylistProvider
 from core.musiq.playback import Playback
-from core.musiq.soundcloud import SoundcloudSongProvider, SoundcloudPlaylistProvider
-from core.musiq.spotify import SpotifySongProvider, SpotifyPlaylistProvider
 from core.musiq.suggestions import Suggestions
-from core.musiq.youtube import YoutubeSongProvider, YoutubePlaylistProvider
 from core.state_handler import Stateful
 
 if TYPE_CHECKING:
@@ -37,6 +34,7 @@ class Musiq(Stateful):
 
     def __init__(self, base: "Base") -> None:
         self.base = base
+        self.urlpatterns: List[URLPattern] = []
 
         self.suggestions = Suggestions(self)
 
@@ -73,15 +71,33 @@ class Musiq(Stateful):
         if playlist:
             music_provider_class = PlaylistProvider
             local_provider_class = PlaylistProvider
-            soundcloud_provider_class = SoundcloudPlaylistProvider
-            spotify_provider_class = SpotifyPlaylistProvider
-            youtube_provider_class = YoutubePlaylistProvider
+            if self.base.settings.platforms.soundcloud_enabled:
+                from core.musiq.soundcloud import SoundcloudPlaylistProvider
+
+                soundcloud_provider_class = SoundcloudPlaylistProvider
+            if self.base.settings.platforms.spotify_enabled:
+                from core.musiq.spotify import SpotifyPlaylistProvider
+
+                spotify_provider_class = SpotifyPlaylistProvider
+            if self.base.settings.platforms.youtube_enabled:
+                from core.musiq.youtube import YoutubePlaylistProvider
+
+                youtube_provider_class = YoutubePlaylistProvider
         else:
             music_provider_class = SongProvider
             local_provider_class = LocalSongProvider
-            soundcloud_provider_class = SoundcloudSongProvider
-            spotify_provider_class = SpotifySongProvider
-            youtube_provider_class = YoutubeSongProvider
+            if self.base.settings.platforms.soundcloud_enabled:
+                from core.musiq.soundcloud import SoundcloudSongProvider
+
+                soundcloud_provider_class = SoundcloudSongProvider
+            if self.base.settings.platforms.spotify_enabled:
+                from core.musiq.spotify import SpotifySongProvider
+
+                spotify_provider_class = SpotifySongProvider
+            if self.base.settings.platforms.youtube_enabled:
+                from core.musiq.youtube import YoutubeSongProvider
+
+                youtube_provider_class = YoutubeSongProvider
 
         if key is not None:
             # an archived entry was requested.
@@ -201,39 +217,18 @@ class Musiq(Stateful):
         provider = SongProvider.create(self, external_url=current_song.external_url)
         return provider.request_radio(request_ip)
 
-    @csrf_exempt
-    def post_song(self, request: WSGIRequest) -> HttpResponse:
-        """This endpoint is part of the API and exempt from CSRF checks.
-        Shareberry uses this endpoint."""
-        # only get ip on user requests
-        if self.base.settings.basic.logging_enabled:
-            request_ip, _ = ipware.get_client_ip(request)
-            if request_ip is None:
-                request_ip = ""
-        else:
-            request_ip = ""
-        query = request.POST.get("query")
-        if not query:
-            return HttpResponseBadRequest("No query to share.")
-        # Set the requested platform to 'spotify'.
-        # It will automatically fall back to Youtube
-        # if Spotify is not enabled or a youtube link was requested.
-        successful, message, _ = self.do_request_music(
-            request_ip, query, None, False, "spotify"
-        )
-        if not successful:
-            return HttpResponseBadRequest(message)
-        return HttpResponse(message)
-
     def index(self, request: WSGIRequest) -> HttpResponse:
         """Renders the /musiq page."""
         context = self.base.context(request)
+        context["urls"] = self.urlpatterns
         context["additional_keywords"] = self.base.settings.basic.additional_keywords
         context["forbidden_keywords"] = self.base.settings.basic.forbidden_keywords
         return render(request, "musiq.html", context)
 
     def state_dict(self) -> Dict[str, Any]:
         state_dict = self.base.state_dict()
+
+        musiq_state = {}
         current_song: Optional[Dict[str, Any]]
         try:
             current_song = model_to_dict(CurrentSong.objects.get())
@@ -255,10 +250,10 @@ class Musiq(Stateful):
                 song_dict["duration"]
             )
             song_queue.append(song_dict)
-        state_dict["total_time_formatted"] = song_utils.format_seconds(total_time)
+        musiq_state["total_time_formatted"] = song_utils.format_seconds(total_time)
 
         if state_dict["alarm"]:
-            state_dict["current_song"] = {
+            musiq_state["current_song"] = {
                 "queue_key": -1,
                 "manually_requested": False,
                 "votes": None,
@@ -270,7 +265,7 @@ class Musiq(Stateful):
                 "created": "",
             }
         elif self.playback.backup_playing.is_set():
-            state_dict["current_song"] = {
+            musiq_state["current_song"] = {
                 "queue_key": -1,
                 "manually_requested": False,
                 "votes": None,
@@ -282,12 +277,14 @@ class Musiq(Stateful):
                 "created": "",
             }
         else:
-            state_dict["current_song"] = current_song
-        state_dict["paused"] = self.playback.paused()
-        state_dict["progress"] = self.playback.progress()
-        state_dict["shuffle"] = self.controller.shuffle
-        state_dict["repeat"] = self.controller.repeat
-        state_dict["autoplay"] = self.controller.autoplay
-        state_dict["volume"] = self.controller.volume
-        state_dict["song_queue"] = song_queue
+            musiq_state["current_song"] = current_song
+        musiq_state["paused"] = self.playback.paused()
+        musiq_state["progress"] = self.playback.progress()
+        musiq_state["shuffle"] = self.controller.shuffle
+        musiq_state["repeat"] = self.controller.repeat
+        musiq_state["autoplay"] = self.controller.autoplay
+        musiq_state["volume"] = self.controller.volume
+        musiq_state["song_queue"] = song_queue
+
+        state_dict["musiq"] = musiq_state
         return state_dict
