@@ -47,18 +47,31 @@ class PlaylistProvider(MusicProvider):
             from core.musiq.localdrive import LocalPlaylistProvider
 
             provider_class = LocalPlaylistProvider
-        elif playlist_type == "youtube":
+        elif (
+            musiq.base.settings.platforms.youtube_enabled and playlist_type == "youtube"
+        ):
             from core.musiq.youtube import YoutubePlaylistProvider
 
             provider_class = YoutubePlaylistProvider
-        elif playlist_type == "spotify":
+        elif (
+            musiq.base.settings.platforms.spotify_enabled and playlist_type == "spotify"
+        ):
             from core.musiq.spotify import SpotifyPlaylistProvider
 
             provider_class = SpotifyPlaylistProvider
-        elif playlist_type == "soundcloud":
+        elif (
+            musiq.base.settings.platforms.soundcloud_enabled
+            and playlist_type == "soundcloud"
+        ):
             from core.musiq.soundcloud import SoundcloudPlaylistProvider
 
             provider_class = SoundcloudPlaylistProvider
+        elif playlist_type == "playlog":
+            # The playlist may contain various song types, but all of them will be archived.
+            # We can use the local playlist provider to enqueue them.
+            from core.musiq.localdrive import LocalPlaylistProvider
+
+            provider_class = LocalPlaylistProvider
         if not provider_class:
             raise NotImplementedError(f"No provider for given playlist: {query}, {key}")
         provider = provider_class(musiq, query, key)
@@ -88,6 +101,7 @@ class PlaylistProvider(MusicProvider):
             except (KeyError, ArchivedPlaylist.DoesNotExist):
                 return False
         self.id = archived_playlist.list_id
+        self.title = archived_playlist.title
         self.key = archived_playlist.id
         self.urls = [entry.url for entry in archived_playlist.entries.all()]
         return True
@@ -174,11 +188,19 @@ class PlaylistProvider(MusicProvider):
             if index == self.musiq.base.settings.basic.max_playlist_items:
                 break
             # request every url in the playlist as their own url
-            song_provider = SongProvider.create(self.musiq, external_url=external_url)
-
             try:
+                song_provider = SongProvider.create(
+                    self.musiq, external_url=external_url
+                )
                 song_provider.request("", archive=False, manually_requested=False)
-            except ProviderError:
+            except (ProviderError, NotImplementedError) as e:
+                logging.warning(
+                    "Error while enqueuing url %s to playlist %s: %s",
+                    external_url,
+                    self.title,
+                    self.id,
+                )
+                logging.exception(e)
                 continue
 
             if settings.DEBUG:
