@@ -11,6 +11,7 @@ from main import settings
 if TYPE_CHECKING:
     from typing_extensions import TypedDict
     from core.musiq.music_provider import ArchivedPlaylist
+    from core.musiq.musiq import Musiq
 
     Metadata = TypedDict(  # pylint: disable=invalid-name
         "Metadata",
@@ -49,6 +50,9 @@ def determine_url_type(url: str) -> str:
 def determine_playlist_type(archived_playlist: "ArchivedPlaylist") -> str:
     """Uses the url of the first song in the playlist
     to determine the platform where the playlist is from."""
+    if archived_playlist.list_id.startswith("playlog"):
+        # The playlist was created from play logs and may contain various song types.
+        return "playlog"
     first_song = archived_playlist.entries.first()
     if not first_song:
         raise ValueError("Playlist contains no songs.")
@@ -67,7 +71,7 @@ def format_seconds(seconds: int) -> str:
 
     formatted = ""
     if hours > 0:
-        formatted += "{:02d}:".format(int(hours))
+        formatted += "{:d}:".format(int(hours))
     formatted += "{0:02d}:{1:02d}".format(int(minutes), int(seconds))
     return formatted
 
@@ -105,12 +109,15 @@ def get_metadata(path: str) -> "Metadata":
     return metadata
 
 
-def contains_keywords(title: str, keywords: str) -> bool:
+def is_forbidden(musiq: "Musiq", s: str) -> bool:
+    # We can't access the variable in settings/basic.py
+    # since we are in a static context without a reference to bes
+    keywords = musiq.base.settings.basic.forbidden_keywords
     words = re.split(r"[,\s]+", keywords.strip())
     # delete empty matches
     words = [word for word in words if word]
 
     for word in words:
-        if re.search(word, title, re.IGNORECASE):
+        if re.search(word, s, re.IGNORECASE):
             return True
     return False
