@@ -14,9 +14,6 @@ from watson import search as watson
 import core.musiq.song_utils as song_utils
 from core.models import ArchivedPlaylist, ArchivedSong
 from core.musiq.song_provider import SongProvider
-from core.musiq.soundcloud import Soundcloud
-from core.musiq.spotify import Spotify
-from core.musiq.youtube import Youtube
 
 if TYPE_CHECKING:
     from core.musiq.musiq import Musiq
@@ -67,8 +64,10 @@ class Suggestions:
                 platform_settings.spotify_enabled
                 and platform_settings.spotify_suggestions > 0
             ):
+                from core.musiq.spotify import Spotify
+
                 spotify_suggestions = Spotify().get_search_suggestions(
-                    query, suggest_playlist
+                    self.musiq, query, suggest_playlist
                 )
                 spotify_suggestions = spotify_suggestions[
                     : platform_settings.spotify_suggestions
@@ -86,7 +85,11 @@ class Suggestions:
                 platform_settings.soundcloud_enabled
                 and platform_settings.soundcloud_suggestions > 0
             ):
-                soundcloud_suggestions = Soundcloud().get_search_suggestions(query)
+                from core.musiq.soundcloud import Soundcloud
+
+                soundcloud_suggestions = Soundcloud().get_search_suggestions(
+                    self.musiq, query
+                )
                 soundcloud_suggestions = soundcloud_suggestions[
                     : platform_settings.soundcloud_suggestions
                 ]
@@ -99,7 +102,11 @@ class Suggestions:
                 platform_settings.youtube_enabled
                 and platform_settings.youtube_suggestions > 0
             ):
-                youtube_suggestions = Youtube().get_search_suggestions(query)
+                from core.musiq.youtube import Youtube
+
+                youtube_suggestions = Youtube().get_search_suggestions(
+                    self.musiq, query
+                )
                 youtube_suggestions = youtube_suggestions[
                     : platform_settings.youtube_suggestions
                 ]
@@ -143,9 +150,20 @@ class Suggestions:
 
             for search_result in search_results:
                 song_info = search_result.meta
-                provider = SongProvider.create(
-                    self.musiq, external_url=song_info["url"]
-                )
+
+                if song_utils.is_forbidden(
+                    self.musiq, song_info["artist"]
+                ) or song_utils.is_forbidden(self.musiq, song_info["title"]):
+                    continue
+
+                try:
+                    provider = SongProvider.create(
+                        self.musiq, external_url=song_info["url"]
+                    )
+                except NotImplementedError:
+                    # For this song a provider is necessary that is not available
+                    # e.g. the song was played before, but the provider was disabled
+                    continue
                 cached = provider.check_cached()
                 # don't suggest local songs if they are not cached (=not at expected location)
                 if not cached and provider.type == "local":
