@@ -7,6 +7,7 @@ import json
 import logging
 import os
 import pickle
+import shutil
 import subprocess
 from contextlib import contextmanager
 from typing import Any, Dict, List, Optional, TYPE_CHECKING, Iterator, cast
@@ -34,6 +35,13 @@ def youtube_session() -> Iterator[requests.Session]:
 
     pickle_file = os.path.join(settings.BASE_DIR, "config/youtube_cookies.pickle")
 
+    session = requests.session()
+    # Have youtube-dl deal with consent cookies etc to setup a valid session
+    extractor = youtube_dl.extractor.youtube.YoutubeIE()
+    extractor._downloader = youtube_dl.YoutubeDL()
+    extractor.initialize()
+    session.cookies.update(extractor._downloader.cookiejar)
+
     try:
         if os.path.getsize(pickle_file) > 0:
             with open(pickle_file, "rb") as f:
@@ -71,6 +79,8 @@ class YoutubeDLLogger:
 class Youtube:
     """This class contains code for both the song and playlist provider"""
 
+    atomicparsley_available = shutil.which("AtomicParsley") is not None
+
     @staticmethod
     def get_ydl_opts() -> Dict[str, Any]:
         """This method returns a dictionary containing sensible defaults for youtube-dl options.
@@ -79,6 +89,15 @@ class Youtube:
             --no-playlist --no-cache-dir --write-thumbnail --default-search auto \
             --add-metadata --embed-thumbnail
         """
+        postprocessors = [{"key": "FFmpegMetadata"}]
+        if Youtube.atomicparsley_available:
+            postprocessors.append(
+                {
+                    "key": "EmbedThumbnail",
+                    # overwrite any thumbnails already present
+                    "already_have_thumbnail": True,
+                }
+            )
         return {
             "format": "bestaudio[ext=m4a]/best[ext=m4a]",
             "outtmpl": os.path.join(settings.SONGS_CACHE_DIR, "%(id)s.%(ext)s"),
@@ -87,15 +106,7 @@ class Youtube:
             "no_color": True,
             "writethumbnail": True,
             "default_search": "auto",
-            "postprocessors": [
-                {"key": "FFmpegMetadata"},
-                # https://github.com/ytdl-org/youtube-dl/pull/25717
-                # {
-                #     "key": "EmbedThumbnail",
-                #     # overwrite any thumbnails already present
-                #     "already_have_thumbnail": True,
-                # },
-            ],
+            "postprocessors": postprocessors,
             "logger": YoutubeDLLogger(),
         }
 
@@ -104,17 +115,16 @@ class Youtube:
         for line in html.split("\n"):
             line = line.strip()
             before = "var ytInitialData = "
-            after = ";</script>"
+            after = ";</"
             if before in line:
                 # extract json
-                initial_data = line[
-                    line.index(before) + len(before) : line.index(after)
-                ]
+                line = line[line.index(before) + len(before) :]
+                initial_data = line[: line.index(after)]
                 return json.loads(initial_data)
         raise ValueError("Could not parse initial data from html")
 
     @staticmethod
-    def get_search_suggestions(query: str) -> List[str]:
+    def get_search_suggestions(musiq: Musiq, query: str) -> List[str]:
         """Returns a list of suggestions for the given query from Youtube."""
         with youtube_session() as session:
             params = {
@@ -130,7 +140,11 @@ class Youtube:
         suggestions = suggestions[1]
         # suggestions are given as tuples
         # extract the string and skip the query if it occurs identically
-        suggestions = [entry[0] for entry in suggestions if entry[0] != query]
+        suggestions = [
+            entry[0]
+            for entry in suggestions
+            if entry[0] != query and not song_utils.is_forbidden(musiq, entry[0])
+        ]
         return suggestions
 
 
@@ -155,29 +169,32 @@ class YoutubeSongProvider(SongProvider, Youtube):
         return os.path.isfile(self._get_path())
 
     def check_available(self) -> bool:
-        try:
-            with youtube_dl.YoutubeDL(self.ydl_opts) as ydl:
-                self.info_dict = ydl.extract_info(self.query, download=False)
-        except (youtube_dl.utils.ExtractorError, youtube_dl.utils.DownloadError) as e:
-            self.error = e
-            return False
 
-        # this value is not an exact match, but it's a good approximation
-        if "entries" in self.info_dict:
-            self.info_dict = self.info_dict["entries"][0]
+        # directly use the search extractors entry function so we can process each result
+        # as soon as it's available instead of waiting for all of them
+        extractor = youtube_dl.extractor.youtube.YoutubeSearchIE()
+        extractor._downloader = youtube_dl.YoutubeDL(self.ydl_opts)
+        extractor.initialize()
+        for entry in extractor._entries(self.query, 50):
+            if song_utils.is_forbidden(self.musiq, entry["title"]):
+                continue
+            try:
+                with youtube_dl.YoutubeDL(self.ydl_opts) as ydl:
+                    self.info_dict = ydl.extract_info(entry["id"], download=False)
+                break
+            except (
+                youtube_dl.utils.ExtractorError,
+                youtube_dl.utils.DownloadError,
+            ) as e:
+                logging.warning("error during availability check for %s:", entry["id"])
+                logging.warning(e)
+        else:
+            self.error = "No songs found"
+            return False
 
         self.id = self.info_dict["id"]
 
-        size = self.info_dict["filesize"]
-        max_size = self.musiq.base.settings.basic.max_download_size * 1024 * 1024
-        if (
-            max_size != 0
-            and self.check_cached() is None
-            and (size is not None and size > max_size)
-        ):
-            self.error = "Song too long"
-            return False
-        return True
+        return self.check_not_too_large(self.info_dict["filesize"])
 
     def _download(self) -> bool:
         error = None
@@ -259,15 +276,11 @@ class YoutubeSongProvider(SongProvider, Youtube):
         path = [
             "contents",
             "twoColumnWatchNextResults",
-            "secondaryResults",
-            "secondaryResults",
-            "results",
+            "autoplay",
+            "autoplay",
+            "sets",
             0,
-            "compactAutoplayRenderer",
-            "contents",
-            0,
-            "compactVideoRenderer",
-            "navigationEndpoint",
+            "autoplayVideo",
             "commandMetadata",
             "webCommandMetadata",
             "url",
@@ -353,15 +366,24 @@ class YoutubePlaylistProvider(PlaylistProvider, Youtube):
         return list_id
 
     def fetch_metadata(self) -> bool:
-        # in case of a radio playist, restrict the number of songs that are downloaded
+        # in case of a radio playlist, restrict the number of songs that are downloaded
         if self.is_radio():
             self.ydl_opts[
                 "playlistend"
             ] = self.musiq.base.settings.basic.max_playlist_items
+            # radios are not viewable with the /playlist?list= url,
+            # create a video watch url with the radio list
+            query_url = (
+                "https://www.youtube.com/watch?v=" + self.id[2:] + "&list=" + self.id
+            )
+        else:
+            # if only given the id, youtube-dl returns an info dict resolving this id to a url.
+            # we want to receive the playlist entries directly, so we query the playlist url
+            query_url = "https://www.youtube.com/playlist?list=" + self.id
 
         try:
             with youtube_dl.YoutubeDL(self.ydl_opts) as ydl:
-                info_dict = ydl.extract_info(self.id, download=False)
+                info_dict = ydl.extract_info(query_url, download=False)
         except (youtube_dl.utils.ExtractorError, youtube_dl.utils.DownloadError) as e:
             self.error = e
             return False
