@@ -11,19 +11,18 @@ from django.http import HttpResponse
 from django.http import HttpResponseBadRequest
 from django.http import JsonResponse
 
+from core import util
 from core.models import Setting
 from core.settings.settings import Settings
 from core.util import background_thread
 
-if TYPE_CHECKING:
-    from core.base import Base
-
 
 class Sound:
     """This class is responsible for handling settings changes related to sound output."""
 
-    def __init__(self, base: "Base"):
-        self.base = base
+    def __init__(self, settings: Settings):
+        self.settings = settings
+        self.output = Settings.get_setting("sound_output", "")
         self.backup_stream = Settings.get_setting("backup_stream", "")
         self.bluetoothctl: Optional[subprocess.Popen[bytes]] = None
         self.bluetooth_devices: List[Dict[str, str]] = []
@@ -38,7 +37,7 @@ class Sound:
     @Settings.option
     def set_backup_stream(self, request: WSGIRequest) -> None:
         """Sets the given internet stream as backup stream."""
-        stream = request.POST.get("stream")
+        stream = request.POST.get("value")
         Setting.objects.filter(key="backup_stream").update(value=stream)
         self.backup_stream = stream
 
@@ -77,30 +76,38 @@ class Sound:
             self.bluetoothctl.stdin.write(b"devices\n")
             self.bluetoothctl.stdin.write(b"scan on\n")
             self.bluetoothctl.stdin.flush()
-            while True:
-                line = self._get_bluetoothctl_line()
-                if not line:
-                    break
-                # match old devices
-                match = re.match(r"Device (\S*) (.*)", line)
-                # match newly scanned devices
-                # We need the '.*' at the beginning of the line to account for control sequences
-                if not match:
-                    match = re.match(r".*\[NEW\] Device (\S*) (.*)", line)
-                if match:
-                    address = match.group(1)
-                    name = match.group(2)
-                    # filter unnamed devices
-                    # devices named after their address are no speakers
-                    if re.match("[A-Z0-9][A-Z0-9](-[A-Z0-9][A-Z0-9]){5}", name):
-                        continue
-                    self.bluetooth_devices.append({"address": address, "name": name})
-                    self.base.settings.update_state()
+
+            @background_thread
+            def do_scan():
+                while True:
+                    line = self._get_bluetoothctl_line()
+                    if not line:
+                        break
+                    # match old devices
+                    match = re.match(r"Device (\S*) (.*)", line)
+                    # match newly scanned devices
+                    # We need the '.*' at the beginning of the line to account for control sequences
+                    if not match:
+                        match = re.match(r".*\[NEW\] Device (\S*) (.*)", line)
+                    if match:
+                        address = match.group(1)
+                        name = match.group(2)
+                        # filter unnamed devices
+                        # devices named after their address are no speakers
+                        if re.match("[A-Z0-9][A-Z0-9](-[A-Z0-9][A-Z0-9]){5}", name):
+                            continue
+                        self.bluetooth_devices.append(
+                            {"address": address, "name": name}
+                        )
+                        self.settings.update_state()
+
+            do_scan()
+            return HttpResponse("Started scanning")
         else:
             if self.bluetoothctl is None:
                 return HttpResponseBadRequest("Currently not scanning")
             self._stop_bluetoothctl()
-        return HttpResponse()
+            return HttpResponse("Stopped scanning")
 
     @Settings.option
     def connect_bluetooth(self, request: WSGIRequest) -> HttpResponse:
@@ -126,23 +133,24 @@ class Sound:
             if self.bluetoothctl is not None:
                 self._stop_bluetoothctl()
 
-        self.bluetoothctl.stdin.write(b"pair " + address.encode() + b"\n")
-        self.bluetoothctl.stdin.flush()
-        while True:
-            line = self._get_bluetoothctl_line()
-            if not line:
-                break
-            if re.match(".*Device " + address + " not available", line):
-                error = "Device unavailable"
-                break
-            if re.match(".*Failed to pair: org.bluez.Error.AlreadyExists", line):
-                break
-            if re.match(".*Pairing successful", line):
-                break
+        # Sometimes, pairing hangs forever. Since connecting alone is enough, skip pairing.
+        # self.bluetoothctl.stdin.write(b"pair " + address.encode() + b"\n")
+        # self.bluetoothctl.stdin.flush()
+        # while True:
+        #     line = self._get_bluetoothctl_line()
+        #     if not line:
+        #         break
+        #     if re.match(".*Device " + address + " not available", line):
+        #         error = "Device unavailable"
+        #         break
+        #     if re.match(".*Failed to pair: org.bluez.Error.AlreadyExists", line):
+        #         break
+        #     if re.match(".*Pairing successful", line):
+        #         break
 
-        if error:
-            self._stop_bluetoothctl()
-            return HttpResponseBadRequest(error)
+        # if error:
+        #     self._stop_bluetoothctl()
+        #     return HttpResponseBadRequest(error)
 
         self.bluetoothctl.stdin.write(b"connect " + address.encode() + b"\n")
         self.bluetoothctl.stdin.flush()
@@ -205,7 +213,7 @@ class Sound:
         return HttpResponse("Disconnected")
 
     @Settings.option
-    def output_devices(self, _request: WSGIRequest) -> JsonResponse:
+    def list_outputs(self, _request: WSGIRequest) -> JsonResponse:
         """Returns a list of all sound output devices currently available."""
         output = subprocess.check_output(
             "pactl list short sinks".split(),
@@ -213,28 +221,63 @@ class Sound:
             universal_newlines=True,
         )
         tokenized_lines = [line.split() for line in output.splitlines()]
-        sinks = [sink[1] for sink in tokenized_lines if len(sink) >= 2]
+
+        sinks = ["icecast", "snapcast"]
+        sinks.extend([sink[1] for sink in tokenized_lines if len(sink) >= 2])
+
         return JsonResponse(sinks, safe=False)
 
-    @Settings.option
-    def set_output_device(self, request: WSGIRequest) -> HttpResponse:
-        """Sets the given device as default output device."""
-        device = request.POST.get("device")
-        if not device:
-            return HttpResponseBadRequest("No device selected")
+    def _set_output(self, output: str) -> HttpResponse:
+        icecast_installed = util.service_installed("icecast2")
+        snapcast_installed = util.service_installed("snapserver")
+
+        if output == "icecast":
+            if not icecast_installed:
+                return HttpResponseBadRequest("Please install icecast2")
+
+            subprocess.call(["sudo", "/usr/local/sbin/raveberry/enable_icecast"])
+            mopidy_output = "icecast"
+        elif output == "snapcast":
+            if not snapcast_installed:
+                return HttpResponseBadRequest("Please install snapserver")
+
+            subprocess.call(["sudo", "/usr/local/sbin/raveberry/enable_snapcast"])
+            mopidy_output = "snapcast"
+        else:
+            try:
+                subprocess.run(
+                    ["pactl", "set-default-sink", output],
+                    stdout=subprocess.DEVNULL,
+                    stderr=subprocess.PIPE,
+                    env={"PULSE_SERVER": "127.0.0.1"},
+                    check=True,
+                )
+                mopidy_output = "pulse"
+            except subprocess.CalledProcessError as e:
+                return HttpResponseBadRequest(e.stderr)
+
+        if icecast_installed and output != "icecast":
+            subprocess.call(["sudo", "/usr/local/sbin/raveberry/disable_icecast"])
+        if snapcast_installed and output != "snapcast":
+            subprocess.call(["sudo", "/usr/local/sbin/raveberry/disable_snapcast"])
+
+        self.settings.system.update_mopidy_config(mopidy_output)
 
-        try:
-            subprocess.run(
-                ["pactl", "set-default-sink", device],
-                stdout=subprocess.DEVNULL,
-                stderr=subprocess.PIPE,
-                env={"PULSE_SERVER": "127.0.0.1"},
-                check=True,
-            )
-        except subprocess.CalledProcessError as e:
-            return HttpResponseBadRequest(e.stderr)
-        # restart mopidy to apply audio device change
-        subprocess.call(["sudo", "/usr/local/sbin/raveberry/restart_mopidy"])
         return HttpResponse(
-            f"Set default output. Restarting the current song might be necessary."
+            "Output was set. Restarting the current song might be necessary."
         )
+
+    @Settings.option
+    def set_output(self, request: WSGIRequest) -> HttpResponse:
+        """Sets the given device as default output device."""
+        output = request.POST.get("value")
+        if not output:
+            return HttpResponseBadRequest("No output selected")
+
+        if output == self.output:
+            return HttpResponseBadRequest("Output unchanged")
+
+        Setting.objects.filter(key="sound_output").update(value=output)
+        self.output = output
+
+        return self._set_output(output)
