@@ -7,8 +7,9 @@ import re
 import shutil
 import subprocess
 import time
-from typing import Dict, TYPE_CHECKING, Tuple, Optional, List
+from typing import Dict, Tuple, Optional
 
+import cachetools.func
 import requests
 from django.conf import settings
 from django.core.handlers.wsgi import WSGIRequest
@@ -16,34 +17,24 @@ from django.http import HttpResponse, JsonResponse
 from django.http import HttpResponseBadRequest
 
 from core.settings.settings import Settings
-from core.util import background_thread
-
-if TYPE_CHECKING:
-    from core.base import Base
 
 
 class System:
     """This class is responsible for handling settings changes related to system configuration."""
 
     @staticmethod
-    def _get_mopidy_config() -> str:
-        if shutil.which("cava"):
-            # if cava is installed, use the visualization config for mopidy
-            config_file = os.path.join(settings.BASE_DIR, "setup/mopidy_cava.conf")
-        else:
-            config_file = os.path.join(settings.BASE_DIR, "setup/mopidy.conf")
-        return config_file
-
-    @staticmethod
-    def update_mopidy_config(config_file=None,) -> None:
+    def update_mopidy_config(output: str) -> None:
         """Updates mopidy's config with the credentials stored in the database.
         If no config_file is given, the default one is used."""
         if settings.DOCKER:
             # raveberry cannot restart mopidy in the docker setup
             return
 
-        if config_file is None:
-            config_file = System._get_mopidy_config()
+        if output == "pulse":
+            if shutil.which("cava"):
+                output = "cava"
+            else:
+                output = "regular"
 
         spotify_username = Settings.get_setting("spotify_username", "")
         spotify_password = Settings.get_setting("spotify_password", "")
@@ -55,7 +46,7 @@ class System:
             [
                 "sudo",
                 "/usr/local/sbin/raveberry/update_mopidy_config",
-                config_file,
+                output,
                 spotify_username,
                 spotify_password,
                 spotify_client_id,
@@ -63,11 +54,10 @@ class System:
                 soundcloud_auth_token,
             ]
         )
-        subprocess.call(["sudo", "/usr/local/sbin/raveberry/restart_mopidy"])
         time.sleep(3)
 
-    def __init__(self, base: "Base"):
-        self.base = base
+    def __init__(self, settings: Settings):
+        self.settings = settings
 
     def check_mopidy_extensions(self) -> Dict[str, Tuple[bool, str]]:
         """Returns a dict indicating for each extension whether it is enabled
@@ -93,10 +83,12 @@ class System:
         parser = configparser.ConfigParser()
         parser.read_string(config)
         extensions = {}
-        for extension in ["spotify", "enabled"]:
+        for extension in ["spotify", "soundcloud"]:
             try:
                 if parser[extension]["enabled"] == "true":
                     extensions[extension] = (True, "Extension probably functional")
+                else:
+                    extensions[extension] = (False, "Extension disabled")
             except KeyError:
                 extensions[extension] = (False, "Extension disabled")
         return extensions
@@ -159,32 +151,6 @@ class System:
                 extensions["soundcloud"] = (True, "No info found, enabling te be safe")
         return extensions
 
-    @Settings.option
-    def enable_streaming(self, _request: WSGIRequest) -> HttpResponse:
-        """Enable icecast streaming."""
-        icecast_exists = False
-        for line in subprocess.check_output(
-            "systemctl list-unit-files --full --all".split(), universal_newlines=True
-        ).splitlines():
-            if "icecast2.service" in line:
-                icecast_exists = True
-                break
-
-        if not icecast_exists:
-            return HttpResponseBadRequest("Please install icecast2")
-
-        subprocess.call(["sudo", "/usr/local/sbin/raveberry/enable_streaming"])
-        config_file = os.path.join(settings.BASE_DIR, "setup/mopidy_icecast.conf")
-        self.update_mopidy_config(config_file)
-        return HttpResponse()
-
-    @Settings.option
-    def disable_streaming(self, _request: WSGIRequest) -> HttpResponse:
-        """Disable icecast streaming."""
-        subprocess.call(["sudo", "/usr/local/sbin/raveberry/disable_streaming"])
-        self.update_mopidy_config()
-        return HttpResponse()
-
     @Settings.option
     def disable_events(self, _request: WSGIRequest) -> None:
         """Disable websocket support."""
@@ -206,15 +172,15 @@ class System:
         subprocess.call(["sudo", "/usr/local/sbin/raveberry/enable_hotspot"])
 
     @Settings.option
-    def unprotect_wifi(self, _request: WSGIRequest) -> None:
+    def disable_wifi_protection(self, _request: WSGIRequest) -> None:
         """Disable password protection of the hotspot, making it public."""
-        subprocess.call(["sudo", "/usr/local/sbin/raveberry/unprotect_wifi"])
+        subprocess.call(["sudo", "/usr/local/sbin/raveberry/disable_wifi_protection"])
 
     @Settings.option
-    def protect_wifi(self, _request: WSGIRequest) -> None:
+    def enable_wifi_protection(self, _request: WSGIRequest) -> None:
         """Enable password protection of the hotspot.
         The password was defined during setup."""
-        subprocess.call(["sudo", "/usr/local/sbin/raveberry/protect_wifi"])
+        subprocess.call(["sudo", "/usr/local/sbin/raveberry/enable_wifi_protection"])
 
     @Settings.option
     def disable_tunneling(self, _request: WSGIRequest) -> None:
@@ -255,33 +221,35 @@ class System:
         subprocess.call(["sudo", "/usr/local/sbin/raveberry/shutdown_system"])
 
     @classmethod
+    @cachetools.func.ttl_cache(ttl=60 * 60 * 24)
     def _fetch_latest_version(cls) -> Optional[str]:
         """Looks up the newest version number from PyPi and returns it."""
-        try:
-            subprocess.run(
+        # https://github.com/pypa/pip/issues/9139
+        p = subprocess.run(
+            "pip3 install --use-deprecated=legacy-resolver raveberry==nonexistingversion".split(),
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.PIPE,
+            universal_newlines=True,
+        )
+        if p.returncode == 2:
+            # pip does not now the --use-deprecated=legacy-resolver
+            # probably an older version that does not need it
+            p = subprocess.run(
                 "pip3 install raveberry==nonexistingversion".split(),
                 stdout=subprocess.DEVNULL,
                 stderr=subprocess.PIPE,
                 universal_newlines=True,
-                check=True,
             )
-        except subprocess.CalledProcessError as e:
-            # parse the newest verson from pip output
-            for line in e.stderr.splitlines():
-                if "from versions" in line:
-                    versions = [re.sub(r"[^0-9.]", "", token) for token in line.split()]
-                    versions = [version for version in versions if version]
-                    latest_version = versions[-1]
-                    return latest_version
-        return None
 
-    @Settings.option
-    def upgrade_available(self, _request: WSGIRequest) -> HttpResponse:
-        latest_version = self._fetch_latest_version()
-        current_version = settings.VERSION
-        if latest_version != current_version:
-            return JsonResponse(True, safe=False)
-        return JsonResponse(False, safe=False)
+        # parse the newest verson from pip output
+        for line in p.stderr.splitlines():
+            if "from versions" in line:
+                versions = [re.sub(r"[^0-9.]", "", token) for token in line.split()]
+                versions = [version for version in versions if version]
+                latest_version = versions[-1]
+                return latest_version
+        else:
+            return None
 
     @Settings.option
     def get_latest_version(self, _request: WSGIRequest) -> HttpResponse:
@@ -302,7 +270,7 @@ class System:
     @Settings.option
     def get_upgrade_config(self, _request: WSGIRequest) -> HttpResponse:
         """Returns the config that will be used for the upgrade."""
-        with open(os.path.join(settings.BASE_DIR, "config/raveberry.ini")) as f:
+        with open(os.path.join(settings.BASE_DIR, "config/raveberry.yaml")) as f:
             config = f.read()
         lines = config.splitlines()
         lines = [line for line in lines if not line.startswith("#")]
@@ -312,16 +280,6 @@ class System:
     def upgrade_raveberry(self, _request: WSGIRequest) -> HttpResponse:
         """Performs an upgrade of raveberry."""
 
-        @background_thread
-        def do_upgrade() -> None:
-            subprocess.call(
-                [
-                    "sudo",
-                    "/usr/local/sbin/raveberry/upgrade_raveberry",
-                    os.path.join(settings.BASE_DIR, "config/raveberry.ini"),
-                ]
-            )
-
-        do_upgrade()
+        subprocess.call(["sudo", "/usr/local/sbin/raveberry/start_upgrade_service"])
 
         return HttpResponse("Upgrading... Look for logs in /var/www/")
